
/*Copyright (c) 1996 Akinwale O. Olugbile, Case Western Reserve University. Released under conditions of the GPL.
All Right Reserved.(c)
See the file COPYING for details.
*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <netdb.h>
#include <gsm.h>
#include <compression.h>
#include <Copyright.h>
extern int optind;
extern char * optarg;

main(int argc, char *argv[])
{
	int ld;
	FILE *audio;	
	struct sockaddr_in name;
	struct in_addr ifaddr, grpaddr;	/* interface address and multicast group address*/
        struct ip_mreq mreq;	/* multicast request address */
	struct hostent *hptr, *gethostbyname();
	int namelen = sizeof(name);
	char aufile[254] = "/dev/audio";

	char *GRPADDR[]  = { "224.0.07.1"
				,"224.0.07.2"
				,"224.0.07.3"
				,"224.0.07.4"
				,"224.0.07.5"
				,"224.0.07.6"
				,"224.0.07.7"
				,"224.0.07.8"
				,"224.0.07.9"
				,"224.0.07.10"
	}; 

	int port = 2122;
	int op;

	int first=1;
	int channel = 0;
	int verbose = 0;
	int group = 0;

	register int i;
	register int eot = 0;
	register int seq = 0;
	register int temp;
	register int max;
	register int missing = 0;
	register int misorder = 0;
	register int current;

	gsm handle;
	s_buffer buf;
	gsm_signal sample[160];

#define OPTIONS "hgvf:p:c:"
#define USAGE "usage: %s [-hgv] [-p port] [-c channel (0 - 9)] \n\n\n -h\tThis help. \n -g\tTurn conferencing mode ON (default OFF). \n -v\tVerbose mode. \n -f\tUse speceified output file (default /dev/audio). \n -p\t<Port number> > = 1024 (default 2122). \n -c\tConference channel number 0 - 9 (default 0). \n\n"
	int buffersize = sizeof(buf);
	int samplesize = sizeof(sample[160]);

	while((op = getopt(argc, argv, OPTIONS)) !=EOF) { 

		switch(op) {
			default:
			case 'h':
				printf(copyright);
				printf(USAGE, argv[0]);
				exit(1);
				break;

			case 'p':
				port=atoi(optarg);
				break;

			case 'c':
				channel = atoi(optarg);
				break;
			
			case 'g':
				group++;
				break;
		
			case 'v':
				verbose++;
				break;

			case 'f':
				memcpy(aufile, optarg, sizeof(aufile));
				break;

		}

	} 

	if (( ld = socket(AF_INET,SOCK_DGRAM, 0))< 0 ) {
		perror("socket");
		exit(1);
	}

	name.sin_addr.s_addr=INADDR_ANY;
	name.sin_family=AF_INET;
	name.sin_port = htons(port);
	
	if (verbose)
		fprintf(stderr,"\n\tPort: %d\n\tSample size: %d\n\tFrame size: %d", ntohs(name.sin_port), samplesize, buffersize);

	if (bind (ld, (struct sockaddr *)&name, sizeof (name)) < 0) {
                perror("bind");
		exit(2);
	}

#ifdef IP_ADD_MEMBERSHIP

	if (group) {

		if ((name.sin_addr.s_addr = (unsigned long) inet_addr(GRPADDR[channel])) == -1) {
			if ((hptr = gethostbyname(GRPADDR[channel])) == NULL) {
				perror("gethostbyname");
				exit(1);
			}

			else memcpy( &name.sin_addr.s_addr, hptr->h_addr, hptr->h_length);
		}

		/*****Here is multicasting******/
		grpaddr.s_addr = name.sin_addr.s_addr;
		ifaddr.s_addr = htonl(INADDR_ANY);
		mreq.imr_multiaddr = grpaddr;
		mreq.imr_interface = ifaddr;

		/* socket opt to become a member of Multicast group */
		if (setsockopt(ld, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&mreq, sizeof(mreq)) < 0) {
			perror("setsockopt");
			exit(1);
		}

		if (verbose) {
			fprintf(stderr,"\n\tConference mode: ON");
			fprintf(stderr,"\n\tReceiving from host(s) on channel: %d",channel);
		}

	} else {

		if (verbose) {
			fprintf(stderr,"\n\tConference mode: OFF");
			fprintf(stderr,"\n\tReceiving from all hosts");
		}

	}

#else 
	fprintf(stderr,"\n\tUnable to initiate Conference: mulicast support unavailable.");
	exit(2);
#endif

	audio=fopen(aufile,"wb");
	
	if (audio == NULL) {
		fprintf(stderr,"\nUnable to open audio file, %s\n",aufile);
		exit(1);
	}

	if (! (handle = gsm_create())) {
		perror("Unable to create gsm_handle");
		exit(1);
	}

	fprintf(stderr,"\n\n\t\tRunning...");	
	
	fflush(audio);
	
	max = MAXSEQ;

	while(!eot) {

		recvfrom(ld, (char*)&buf, buffersize,  0, NULL, NULL); 

		if (buf.encoded) {

			if (gsm_decode(handle, buf.buffer, sample) < 0) {
				perror("Error Decoding");
				exit(2);
			} 

			for(i = 0; i < 160; i++) 
				fputc( (char) S2U( (unsigned short) sample[i] ), audio);

			if (verbose) {

				if (temp !=buf.encoded) {
					fprintf(stderr,"\n\tEncoding ON");
					temp = buf.encoded;
				}

			}

		} else { 
			fwrite(buf.buffer, sizeof(buf.buffer), 1, audio);
			
			if (verbose) {
	
				if (temp !=buf.encoded) {
					fprintf(stderr,"\n\tEncoding oFF");
					temp = buf.encoded;
				}

			}

		} 
		
		eot = buf.eot;
		temp = buf.encoded;
		current = ntohs(buf.seq);

		if (verbose) {

			if ( seq != current ) {
				if (seq > current) { 
					misorder++; 
					if (missing)
						missing--;
				} else
					missing++;

				fprintf(stderr,"\n\t%d Packets dropped \n\t%d Packets misordered", missing, misorder );
			}

			seq = current; 

			if (seq == max)
				seq = 0;
			else seq++;

		}

	}

	fprintf(stderr,"\n\n");
}
