/*Copyright (c) 1996 Akinwale O. Olugbile, Case Western Reserve University. Released under conditions of the GPL.
All Right Reserved.(c)
See the file COPYING for details.
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <netdb.h>
#include <gsm.h>
#include <compression.h>
#include <Copyright.h>
	
extern int optind;
extern char * optarg;

main(int argc, char *argv[])
{
	
	int ld;
	int addrlen;
	int fd;
	FILE *audio;	
	struct sockaddr_in name;
	struct hostent *hptr, *gethostbyname();

	int namelen = sizeof(name);
	char destination[256];
	char aufile[254] = "/dev/audio";

	/* flags */
	int compression = 1;
	int group = 0;
	int dest = 1;
	int verbose = 0;
	int channel = 0;

	int port = htons(2122);
	u_char ttl = 1;
	int op;

	register int i;
	register int max;
	register int seq = 0;

 	gsm handle;
	s_buffer buf;
	gsm_signal sample[160];

	char *GRPADDR[] = { "224.0.07.01"
				,"224.0.07.02"
				,"224.0.07.03"
				,"224.0.07.04"
				,"224.0.07.05"
				,"224.0.07.06"
				,"224.0.07.07"
				,"224.0.07.08"
				,"224.0.07.09"
				,"224.0.07.10"
	};

#define OPTIONS "hvbgf:c:p:d:"
#define USAGE "Usage: %s [-hbgv] [-f audio file] [-c channel (0 - 9)] [-p port] [-d destination] \n\n\n -h\tThis help. \n -b\tTurn compression mode OFF (default ON). \n -g\tTurn conferencing mode ON (default OFF). \n -v\tVerbose mode. \n -f\tUse speceified input file (default /dev/audio). \n -c\tConference channel number 0 - 9 (default 0). \n -p\t<Port number> >= 1024 (default 2122). \n -d\t<Destination>. \n\n"

int buffersize = sizeof(buf);
int samplesize = sizeof(sample[160]);
 
	while((op = getopt(argc, argv, OPTIONS)) !=EOF) { 
		switch(op) {

			default:
			case 'h':
				printf(copyright);
				printf(USAGE, argv[0]);
				exit(1);
				break;

			case 'b':
				compression--;
				break;

			case 'g':
				group++;
				break;

			case 'c':
				channel = atoi(optarg);
	
				if ( channel < 0 | channel > 9) {
					fprintf(stderr, "Invalid channel number");
					exit(1);
				}

				break; 

			case 'v':
				verbose++;
				break;

			case 'p':
				port = htons(atoi(optarg));
				break;

			case 'd':
				memcpy(destination, optarg, sizeof(destination));
				dest--;
				break;	

			case 'f':
				memcpy(aufile, optarg, sizeof(aufile));
				break;

		}
	}

	if (!group & dest) { 
                fprintf(stderr,"Error: No destination specified\n");
		printf(copyright);
                printf(USAGE, argv[0]);
                exit(1);
        }

	if (( ld = socket(AF_INET,SOCK_DGRAM, 0))< 0 ) {
		perror("socket");
		exit(1); 
	}

	name.sin_addr.s_addr = INADDR_ANY;
	name.sin_family = AF_INET;
	name.sin_port = port;

	if(verbose)
		printf("\n\tPort: %d",ntohs(name.sin_port));

	if((bind(ld,(struct sockaddr*)&name,namelen)) < 0) {
		perror("bind");
		exit(1);
	}

	if (group) 
		memcpy(destination, GRPADDR[channel], sizeof(destination) );	

	/* hostname lookup  */ /* destination */
	if ((name.sin_addr.s_addr = (unsigned long) inet_addr(destination)) == -1) {
		if ((hptr = gethostbyname(destination)) == NULL) { 
			perror("gethostbyname");
			exit(1);
		}

		else memcpy( &name.sin_addr.s_addr, hptr->h_addr, hptr->h_length);
	}

	if (group) {

		/*****Here is multicasting******/
		int on = 1;

		/* set sock opt so that me may multicast broadcast */
                if (setsockopt(ld, SOL_SOCKET, SO_BROADCAST, (char *)&on, sizeof (on)) < 0) {
                        perror("setsockopt");
                        close(ld);
                        return -1;
		}
	
		if (setsockopt(ld, IPPROTO_IP, IP_MULTICAST_TTL, (char *)&ttl, sizeof(ttl)) < 0) {
			perror("setsockopt1");
			exit(1);
		 }

		if (verbose)	
			fprintf(stderr,"\n\tGroup Conference mode: ON");

	} else  {

		if (verbose)
			fprintf(stderr,"\n\tGroup Conference mode: OFF");
	}

	if (verbose)
		fprintf(stderr,"\n\tSample size: %d\n\tFrame size: %d", samplesize, buffersize);

	audio = fopen(aufile,"rb");

	if (audio == NULL) {
		fprintf(stderr,"\nUnable to open file, %s\n", aufile);
		exit(1);
	}

	fflush(audio);

	if (compression) {

		if (! (handle = gsm_create())) {

			perror("Unable to create gsm_handle");

			exit(1);
		}

		if (verbose)
			fprintf(stderr,"\n\tCompression mode: ON");

		buf.encoded = 1;

	} else {	

		buf.encoded = 0;

		if (verbose)
			fprintf(stderr,"\n\tCompression mode: OFF");

	}

	if (group)
		fprintf(stderr,"\n\n\tSending %s -> Conference Channel %d", aufile, channel);
	else
		fprintf(stderr,"\n\n\tSending %s -> (%s) ...\n", aufile, inet_ntoa(name.sin_addr));

	max = MAXSEQ;

	while(!feof(audio))	
	{

		if (compression) {

			for ( i = 0; i<160; i++) 
				sample[i]=U2S((unsigned short)fgetc(audio));
	
			gsm_encode(handle, sample, buf.buffer);

		} else 
			fread(buf.buffer, sizeof(buf.buffer), 1, audio);

		buf.seq = seq;
		buf.eot = 0;

		sendto(ld,(char *)&buf,buffersize, 0, (struct sockaddr *) &name, namelen);

		if (seq == max)
			seq = 0;
		else
			seq ++;
	}  
	
	memset(buf,0,buffersize);
	buf.seq = seq;
	buf.eot = 1;
	sendto(ld,(char *)&buf,buffersize, 0, (struct sockaddr *) &name, namelen);

}
